@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package App::MediaWiki2Git;
 {
-  $App::MediaWiki2Git::VERSION = '0.001002';
+  $App::MediaWiki2Git::VERSION = '0.002000';
 }
 
 use Moose;
@@ -19,33 +19,91 @@ App::MediaWiki2Git - copy MediaWiki page history into a Git repository
 
 =head1 VERSION
 
-version 0.001002
+version 0.002000
+
+=head1 SYNOPSIS
+
+ # Set up
+ mkdir pages
+ cd pages
+ git init
+ git commit --allow-empty -m 'initial empty commit'
+ printf "---\nmediawiki:\n  api_url: http://example.com/wiki/api.php\npages:\n  - MainPage\n" > mw2git.yaml
+ git add mw2git.yaml
+ git commit -m 'initial config'
+
+ # Fetch pages
+ mw2git
 
 =head1 DESCRIPTION
 
 This is a workaround for the lack of an "annotate" (aka. "blame")
 feature in the MediaWiki we use locally.
 
-It operates using configuration in and upon a Git repository at the
-current directory.
+It operates using configuration in, and upon a Git repository at, the
+current directory.  There are no options.
 
 
 =head1 CONFIGURATION
 
-By default, it expects F<mw2git.yaml> to exist in the current directory.
+By default, it expects F<mw2git.yaml> to exist in the current
+directory.  Via the (Moose-y) OO interface, it can take configuration
+from elsewhere.
 
 This should contain one hash (dictionary), whose entries are used to
 configure parts of this package.
 
-This configuration may be extended and (atomically) replaced as
-fetching progresses.
+=head2 Keys used
+
+=over 4
+
+=item mediawiki
+
+This is passed to L<MediaWiki::API/new> and should be another hash.
+
+One entry for C<api_url> should be enough.  This is probably
+constructed by replacing the C<index.php> in
+"http://...server.../wiki/index.php" with C<api.php> .
+
+See L<MediaWiki::API> for other options such as proxy control.
+
+=item rvlimit
+
+The MediaWiki C<api.php> limits the maximum number of revisions that
+can be fetched in one query.  This is likely to be 50, 500 or 5000
+depending on context.  This module uses 500 as the default.
+
+=item pages
+
+This list defines the pages to be fetched.
+
+TODO: we could populate 'pages' from a category list at the start of each run.
+
+=item dns_qual
+
+(Optional, for local use only.)  This key is appended to unqualified
+hostnames in F<~/.ssh/ssh-config.yaml>, if you have that file.
+
+=item _page_revs
+
+For internal use.  This hash of pagename to last fetched revision id
+is used to avoid querying the api for previous page revisions.
+
+It is the main reason why the configuration file must be rewritten and
+committed along with the copies of the pages.
+
+=back
+
+=head2 Updating
+
+The configuration is extended in-memory, (atomically) replaced on
+disk, and committed as fetching progresses.
 
 XXX: errors during a run can leave the config out of sync with the committed pages
 so page revisions may get committed again.  One solution would be to
 C<reset --hard> to the last config save commit.  This could be
 automated, at some cost to the principle of least surprise.
 
-
 =cut
 
 has config_filename => (is => 'ro', isa => 'Str', lazy_build => 1);
@@ -95,13 +153,12 @@ sub config_save {
 }
 
 
-=head1 MEDIAWIKI INTERFACE
+=head1 OTHER COMPONENTS
 
-The config key C<mediawiki> should contain a hash suitable for passing
-to L<MediaWiki::API/new>.
+=head2 MediaWiki interface
 
-One entry for C<api_url>, constructed by replacing the C<index.php> in
-"http://...server.../wiki/index.php" with C<api.php> should be enough.
+This is used read-only and anonymously (assumes it does not need to
+log in).
 
 =cut
 
@@ -134,32 +191,42 @@ sub go {
     # There are several ways to dice the revision number fetching.
     # This keeps track of latest revision per-page, in our config.
 
-    foreach my $pagename ($self->pages) {
-        my $q = $mw->api
-          ({ action => 'query',
-             prop => 'revisions',
-             titles => $pagename,
-             rvstartid => $self->page_lastrev($pagename),
-             rvdir => 'newer',
-             rvlimit => $self->rvlimit,
-             rvprop => 'ids|flags|timestamp|user|comment|content' })
-        || $self->_mw_error;
+    my $more;
+    do {
+        $more = 0;
+        foreach my $pagename ($self->pages) {
+            my $rv = $self->page_lastrev($pagename);
+            $rv++ if $rv > 0; # avoid re-fetching the last, for tidiness & quiet
 
-        $self->_save_revs(%{ $q->{query}->{pages} }); # one pair
-    }
+            my $q = $mw->api
+              ({ action => 'query',
+                 prop => 'revisions',
+                 titles => $pagename,
+                 rvstartid => $rv,
+                 rvdir => 'newer',
+                 rvlimit => $self->rvlimit,
+                 rvprop => 'ids|flags|timestamp|user|comment|content' })
+                || $self->_mw_error;
+
+            $self->_save_revs(%{ $q->{query}->{pages} }); # one pair
+            $more ||= $q->{'query-continue'};
+        }
+    } while ($more);
 }
 
 
-=head1 GIT INTERFACE
+=head2 Git interface
 
 Uses L<Git::Repository> to drive Git upon the current directory.
+There is no configuration.
 
 It is assumed that the previous requirement for the existence of the
 configuration file is enough of a sanity check, to prevent messing
 with any other Git repositories' history.
 
-You need to initialise the empty repository directory with a suitable
-configuration file.
+It currently performs only C<git add> and C<git commit> operations,
+but might want to C<git reset --hard> later.  This should probably
+require permission from the configuration.
 
 =cut
 
@@ -171,12 +238,18 @@ sub _build_git {
 }
 
 
-=head1 PAGE TRACKING
+=head2 Page tracking
 
 Configuration lists the pages to fetch, and the last revision fetched
 per page.
 
-TODO: we could populate 'pages' from a category
+The Git author is constructed from the page information, including
+some post-processing to attempt to improve the usefulness of anonymous
+(IP address logged) edits.
+
+The committer and commit timestamp are left to be picked up from the
+environment.  This means that Git commitids will not be reproducible
+between different runs of this code on the same page revisions.
 
 =cut
 
@@ -190,12 +263,13 @@ sub pages {
 
 sub page_lastrev {
     my ($self, $pagename, $new_lastrev) = @_;
-    my $revs = $self->config->{page_revs} ||= {};
+    my $revs = $self->config->{_page_revs} ||= {};
     $revs->{$pagename} = $new_lastrev if defined $new_lastrev;
     return $revs->{$pagename} || 0;
 }
 
 
+# Destructively takes out all the page content
 sub _save_revs {
     my ($self, $pageid, $page) = @_;
 
@@ -230,7 +304,7 @@ sub _save_page {
        $props->{comment} || '',
        Dump($props));
 
-    warn "[$author] $pagename $$props{revid}\n";
+    printf("[%s %s] %s\n", $pagename, $props->{revid}, $author);
     $self->git->run(add => $fn);
     $self->git->run
       (commit => '-q',
@@ -243,20 +317,29 @@ sub _save_page {
 }
 
 
-=head1 HOSTNAME LOOKUP
+=head2 Hostname lookup
 
 When users do not log in, we get their IP address.  When this is a web
 proxy, we learn nothing; but in a company it is often a one-user
 desktop machine.
 
-Do some lookup to make the history more useful.
-
-This can take advantage of the local username-to-hostname mapping I
-maintain for ssh aliases, if the configuration file is present.
+We do a reverse lookup in the DNS (IPv4) to get a hostname.  Results
+are cached during the run and errors are written out as warnings.
 
 Beware that looking up historically-recorded IP addresses against the
 current DNS is likely to generate surprises.
 
+=head2 Hostname to user lookup
+
+You may safely ignore this part of the code.
+
+If the custom username-to-hostname mapping is present, we include in
+the "anonymous" author info the result of a lookup.
+
+This is a mapping I maintain to generate ssh host aliases, to assist
+with internal user support.  The tool using it is small and not (yet)
+published.
+
 =cut
 
 has _ptrcache => (is => 'ro', default => sub { {} });
@@ -299,10 +382,12 @@ sub _build_host2nick {
 
     my $u2h = $cfg[0]{map};
     my %map = reverse %$u2h;
-    my $qualify = $self->config_for('dns_qual');
-    foreach my $h (keys %map) {
+    my $qualify = $self->config_for('dns_qual', undef, '');
+    if ($qualify ne '') {
+      foreach my $h (keys %map) {
         next if $h =~ /\./;
         $map{"$h$qualify"} = $map{$h}; # extra entry for FQDN
+      }
     }
 
     return \%map;
@@ -311,3 +396,15 @@ sub _build_host2nick {
 
 no Moose;
 __PACKAGE__->meta->make_immutable;
+
+
+=head1 AUTHOR
+
+Copyright (C) 2011 Genome Research Limited
+
+Author Matthew Astley L<mca@sanger.ac.uk>
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+=cut
